# this_file: more/mkdocs-plugins/vexy-mkdocs-output-as-input/src/mkdocs_output_as_input/__init__.py
"""MkDocs Output as Input Plugin."""

from mkdocs_output_as_input.plugin import OutputAsInputPlugin

try:
    from mkdocs_output_as_input._version import __version__  # type: ignore[import-untyped]
except ImportError:
    __version__ = "0.0.0+unknown"

__all__ = ["OutputAsInputPlugin"]



# this_file: more/mkdocs-plugins/vexy-mkdocs-output-as-input/tests/test_plugin.py
"""Tests for vexy-mkdocs-output-as-input plugin."""

import tempfile
from pathlib import Path
from unittest.mock import Mock

import pytest

from mkdocs_output_as_input.plugin import OutputAsInputPlugin


class TestOutputAsInputPlugin:
    """Test cases for OutputAsInputPlugin."""

    @pytest.fixture
    def plugin(self):
        """Create a plugin instance."""
        plugin = OutputAsInputPlugin()
        # Initialize plugin configuration with defaults
        plugin.config = {
            "stage_dir": "stage",
            "html_element": "main",
            "target_tag": "article",
            "include_frontmatter": True,
            "preserve_links": False,
            "minify": False,
            "prettify": False,
            "verbose": False,
        }
        return plugin

    @pytest.fixture
    def temp_dir(self):
        """Create a temporary directory."""
        with tempfile.TemporaryDirectory() as td:
            yield Path(td)

    @pytest.fixture
    def mkdocs_config(self, temp_dir):
        """Create a mock MkDocs config."""
        config = {
            "site_dir": str(temp_dir / "site"),
            "docs_dir": str(temp_dir / "docs"),
        }
        # Create directories
        Path(config["site_dir"]).mkdir(parents=True)
        Path(config["docs_dir"]).mkdir(parents=True)
        return config

    def test_default_config(self, plugin):
        """Test plugin with default configuration."""
        assert plugin.config["stage_dir"] == "stage"
        assert plugin.config["html_element"] == "main"
        assert plugin.config["target_tag"] == "article"
        assert plugin.config["include_frontmatter"] is True
        assert plugin.config["verbose"] is False

    def test_on_config_stores_paths(self, plugin, mkdocs_config):
        """Test that on_config stores site and docs directories."""
        plugin.on_config(mkdocs_config)

        assert plugin.site_dir == Path(mkdocs_config["site_dir"])
        assert plugin.docs_dir == Path(mkdocs_config["docs_dir"])

    def test_on_page_read_source_captures_content(self, plugin, mkdocs_config, temp_dir):  # noqa: ARG002
        """Test capturing source content and frontmatter."""
        # Create a source file
        docs_dir = Path(mkdocs_config["docs_dir"])
        src_file = docs_dir / "test.md"
        src_file.write_text("""---
title: Test Page
category: Testing
---

# Test Content

This is test content.
""")

        # Create mock page
        mock_file = Mock()
        mock_file.src_path = "test.md"
        mock_file.abs_src_path = str(src_file)

        page = Mock()
        page.file = mock_file

        # Process the page
        plugin.on_config(mkdocs_config)
        result = plugin.on_page_read_source(page, mkdocs_config)

        assert result is None  # Should return None to let MkDocs continue
        assert "test.md" in plugin.source_files
        assert plugin.source_files["test.md"]["frontmatter"]["title"] == "Test Page"
        assert plugin.source_files["test.md"]["frontmatter"]["category"] == "Testing"

    def test_on_page_read_source_no_frontmatter(self, plugin, mkdocs_config, temp_dir):  # noqa: ARG002
        """Test capturing content without frontmatter."""
        # Create a source file without frontmatter
        docs_dir = Path(mkdocs_config["docs_dir"])
        src_file = docs_dir / "no_fm.md"
        src_file.write_text("# No Frontmatter\n\nJust content.")

        # Create mock page
        mock_file = Mock()
        mock_file.src_path = "no_fm.md"
        mock_file.abs_src_path = str(src_file)

        page = Mock()
        page.file = mock_file

        # Process the page
        plugin.on_config(mkdocs_config)
        plugin.on_page_read_source(page, mkdocs_config)

        assert "no_fm.md" in plugin.source_files
        assert plugin.source_files["no_fm.md"]["frontmatter"] == {}

    def test_on_post_build_creates_cousin_files(self, plugin, mkdocs_config, temp_dir):  # noqa: ARG002
        """Test creating cousin files after build."""
        # Setup directories
        docs_dir = Path(mkdocs_config["docs_dir"])
        site_dir = Path(mkdocs_config["site_dir"])

        # Create source file
        src_file = docs_dir / "test.md"
        src_file.write_text("""---
title: Test Page
---

# Test Content
""")

        # Create corresponding HTML output
        html_dir = site_dir / "test"
        html_dir.mkdir(parents=True)
        html_file = html_dir / "index.html"
        html_file.write_text("""<!DOCTYPE html>
<html>
<head><title>Test</title></head>
<body>
<main class="main-content">
<h1>Test Content</h1>
<p>This is the HTML output.</p>
</main>
</body>
</html>""")

        # Simulate page capture
        plugin.on_config(mkdocs_config)
        plugin.source_files["test.md"] = {
            "frontmatter": {"title": "Test Page"},
            "abs_src_path": str(src_file),
        }

        # Run post build
        plugin.on_post_build(mkdocs_config)

        # Check cousin file was created
        stage_dir = docs_dir.parent / "stage"
        cousin_file = stage_dir / "test.md"

        assert cousin_file.exists()
        content = cousin_file.read_text()

        # Check frontmatter
        assert "---" in content
        assert "title: Test Page" in content

        # Check HTML content was extracted and transformed
        assert '<article class="main-content">' in content
        assert "<h1>Test Content</h1>" in content
        assert "<p>This is the HTML output.</p>" in content

    def test_on_post_build_missing_html(self, plugin, mkdocs_config, temp_dir):
        """Test handling missing HTML output."""
        # Setup
        plugin.on_config(mkdocs_config)
        plugin.source_files["missing.md"] = {
            "frontmatter": {},
            "abs_src_path": str(temp_dir / "missing.md"),
        }

        # Should not raise exception
        plugin.on_post_build(mkdocs_config)

    def test_on_post_build_missing_element(self, plugin, mkdocs_config, temp_dir):
        """Test handling HTML without target element."""
        # Setup directories
        site_dir = Path(mkdocs_config["site_dir"])

        # Create HTML without main element
        html_dir = site_dir / "test"
        html_dir.mkdir(parents=True)
        html_file = html_dir / "index.html"
        html_file.write_text("""<!DOCTYPE html>
<html>
<head><title>Test</title></head>
<body>
<div>No main element here</div>
</body>
</html>""")

        # Setup plugin
        plugin.on_config(mkdocs_config)
        plugin.source_files["test.md"] = {
            "frontmatter": {},
            "abs_src_path": str(temp_dir / "test.md"),
        }

        # Should handle gracefully
        plugin.on_post_build(mkdocs_config)

    def test_custom_config_options(self, plugin, mkdocs_config, temp_dir):
        """Test with custom configuration options."""
        # Set custom config
        plugin.config["stage_dir"] = "custom_stage"
        plugin.config["html_element"] = "div"
        plugin.config["target_tag"] = "section"
        plugin.config["verbose"] = True

        # Setup
        docs_dir = Path(mkdocs_config["docs_dir"])
        site_dir = Path(mkdocs_config["site_dir"])

        # Create HTML with div instead of main
        html_file = site_dir / "test.html"
        html_file.write_text("""<!DOCTYPE html>
<html>
<body>
<div class="content">
<h1>Test</h1>
</div>
</body>
</html>""")

        plugin.on_config(mkdocs_config)
        plugin.source_files["test.md"] = {
            "frontmatter": {},
            "abs_src_path": str(temp_dir / "test.md"),
        }

        plugin.on_post_build(mkdocs_config)

        # Check custom stage dir
        stage_dir = docs_dir.parent / "custom_stage"
        cousin_file = stage_dir / "test.md"

        assert cousin_file.exists()
        content = cousin_file.read_text()

        # Check that div was transformed to section
        assert '<section class="content">' in content
        assert "<h1>Test</h1>" in content

    def test_invalid_yaml_frontmatter(self, plugin, mkdocs_config, temp_dir):  # noqa: ARG002
        """Test handling invalid YAML frontmatter."""
        # Create source with invalid YAML
        docs_dir = Path(mkdocs_config["docs_dir"])
        src_file = docs_dir / "invalid.md"
        src_file.write_text("""---
title: Test
invalid: [unclosed
---

# Content
""")

        # Create mock page
        mock_file = Mock()
        mock_file.src_path = "invalid.md"
        mock_file.abs_src_path = str(src_file)

        page = Mock()
        page.file = mock_file

        # Should handle gracefully
        plugin.on_config(mkdocs_config)
        plugin.on_page_read_source(page, mkdocs_config)

        assert "invalid.md" in plugin.source_files
        assert plugin.source_files["invalid.md"]["frontmatter"] == {}

    def test_file_read_error(self, plugin, mkdocs_config):
        """Test handling file read errors."""
        # Create mock page with non-existent file
        mock_file = Mock()
        mock_file.src_path = "nonexistent.md"
        mock_file.abs_src_path = "/path/to/nonexistent.md"

        page = Mock()
        page.file = mock_file

        # Should return None and log error
        plugin.on_config(mkdocs_config)
        result = plugin.on_page_read_source(page, mkdocs_config)

        assert result is None
        assert "nonexistent.md" not in plugin.source_files

    def test_on_post_build_no_docs_dir(self, plugin):
        """Test handling when docs_dir is None."""
        plugin.docs_dir = None
        plugin.on_post_build({})
        # Should handle gracefully without error

    def test_process_file_no_site_dir(self, plugin, temp_dir):
        """Test handling when site_dir is None."""
        plugin.site_dir = None
        stage_dir = temp_dir / "stage"
        stage_dir.mkdir()
        
        file_info = {"frontmatter": {}, "abs_src_path": str(temp_dir / "test.md")}
        plugin._process_file("test.md", file_info, stage_dir)
        # Should handle gracefully without error

    def test_include_frontmatter_option(self, plugin, mkdocs_config, temp_dir):
        """Test include_frontmatter configuration option."""
        docs_dir = Path(mkdocs_config["docs_dir"])
        site_dir = Path(mkdocs_config["site_dir"])

        # Create HTML output
        html_file = site_dir / "test.html"
        html_file.write_text("""<!DOCTYPE html>
<html>
<body>
<main>
<h1>Test Page</h1>
</main>
</body>
</html>""")

        plugin.on_config(mkdocs_config)
        
        # Test with include_frontmatter=True (default)
        plugin.source_files["test.md"] = {
            "frontmatter": {"title": "Test Title", "author": "Test Author"},
            "abs_src_path": str(temp_dir / "test.md"),
        }
        plugin.on_post_build(mkdocs_config)
        
        stage_dir = docs_dir.parent / "stage"
        cousin_file = stage_dir / "test.md"
        content = cousin_file.read_text()
        
        # Should include frontmatter
        assert "---" in content
        assert "title: Test Title" in content
        assert "author: Test Author" in content
        
        # Test with include_frontmatter=False
        plugin.config["include_frontmatter"] = False
        plugin.on_post_build(mkdocs_config)
        
        content = cousin_file.read_text()
        
        # Should NOT include frontmatter
        assert not content.startswith("---")
        assert "title: Test Title" not in content
        assert "author: Test Author" not in content
        # But should still have HTML content
        assert "<article>" in content
        assert "<h1>Test Page</h1>" in content

    def test_preserve_links_option(self, plugin, mkdocs_config, temp_dir):
        """Test preserve_links configuration option."""
        docs_dir = Path(mkdocs_config["docs_dir"])
        site_dir = Path(mkdocs_config["site_dir"])

        # Create HTML output with absolute links
        html_file = site_dir / "test.html"
        html_file.write_text("""<!DOCTYPE html>
<html>
<body>
<main>
<h1>Test Page</h1>
<a href="/docs/page.html">Link</a>
<img src="/images/pic.png">
<a href="https://example.com">External</a>
</main>
</body>
</html>""")

        plugin.on_config(mkdocs_config)
        plugin.source_files["test.md"] = {
            "frontmatter": {},
            "abs_src_path": str(temp_dir / "test.md"),
        }
        
        # Test with preserve_links=True
        plugin.config["preserve_links"] = True
        plugin.on_post_build(mkdocs_config)
        
        stage_dir = docs_dir.parent / "stage"
        cousin_file = stage_dir / "test.md"
        content = cousin_file.read_text()
        
        # Should convert absolute links to relative
        assert 'href="./docs/page.html"' in content
        assert 'src="./images/pic.png"' in content
        # External links should remain unchanged
        assert 'href="https://example.com"' in content

    def test_multiple_selectors(self, plugin, mkdocs_config, temp_dir):
        """Test extraction of multiple selectors."""
        docs_dir = Path(mkdocs_config["docs_dir"])
        site_dir = Path(mkdocs_config["site_dir"])

        # Create HTML with multiple elements
        html_file = site_dir / "test.html"
        html_file.write_text("""<!DOCTYPE html>
<html>
<body>
<header><h1>Header Content</h1></header>
<main><p>Main Content</p></main>
<aside><p>Sidebar Content</p></aside>
<footer><p>Footer Content</p></footer>
</body>
</html>""")

        plugin.on_config(mkdocs_config)
        plugin.source_files["test.md"] = {
            "frontmatter": {},
            "abs_src_path": str(temp_dir / "test.md"),
        }
        
        # Test with multiple selectors
        plugin.config["html_element"] = ["main", "aside"]
        plugin.on_post_build(mkdocs_config)
        
        stage_dir = docs_dir.parent / "stage"
        cousin_file = stage_dir / "test.md"
        content = cousin_file.read_text()
        
        # Should include both main and aside content wrapped in article
        assert "<article>" in content
        assert "Main Content" in content
        assert "Sidebar Content" in content
        # Should NOT include header or footer
        assert "Header Content" not in content
        assert "Footer Content" not in content

    def test_css_selectors(self, plugin, mkdocs_config, temp_dir):
        """Test CSS selector support."""
        docs_dir = Path(mkdocs_config["docs_dir"])
        site_dir = Path(mkdocs_config["site_dir"])

        # Create HTML with CSS selectable elements
        html_file = site_dir / "test.html"
        html_file.write_text("""<!DOCTYPE html>
<html>
<body>
<div class="content">
<h1>Title</h1>
<p class="highlight">Important</p>
<p>Normal</p>
</div>
<div class="sidebar">Sidebar</div>
</body>
</html>""")

        plugin.on_config(mkdocs_config)
        plugin.source_files["test.md"] = {
            "frontmatter": {},
            "abs_src_path": str(temp_dir / "test.md"),
        }
        
        # Test with CSS selector
        plugin.config["html_element"] = ".content"
        plugin.on_post_build(mkdocs_config)
        
        stage_dir = docs_dir.parent / "stage"
        cousin_file = stage_dir / "test.md"
        content = cousin_file.read_text()
        
        # Should extract div with class="content"
        assert "<article" in content
        assert "Title" in content
        assert "Important" in content
        assert "Normal" in content
        # Should NOT include sidebar
        assert "Sidebar" not in content

    def test_minify_option(self, plugin, mkdocs_config, temp_dir):
        """Test minify option."""
        docs_dir = Path(mkdocs_config["docs_dir"])
        site_dir = Path(mkdocs_config["site_dir"])

        # Create HTML with whitespace
        html_file = site_dir / "test.html"
        html_file.write_text("""<!DOCTYPE html>
<html>
<body>
<main>
    <h1>Title</h1>
    <p>
        Content with
        lots of whitespace
    </p>
</main>
</body>
</html>""")

        plugin.on_config(mkdocs_config)
        plugin.source_files["test.md"] = {
            "frontmatter": {},
            "abs_src_path": str(temp_dir / "test.md"),
        }
        
        # Test with minify=True
        plugin.config["minify"] = True
        plugin.on_post_build(mkdocs_config)
        
        stage_dir = docs_dir.parent / "stage"
        cousin_file = stage_dir / "test.md"
        content = cousin_file.read_text()
        
        # Should have no newlines in HTML (except final newline)
        lines = content.strip().split('\n')
        assert len(lines) == 1  # Only one line of HTML
        # Check that spaces between tags are removed
        assert "><" in content
        # Check that content is minified but preserves single spaces in text
        assert "Content with lots of whitespace" in content

    def test_prettify_option(self, plugin, mkdocs_config, temp_dir):
        """Test prettify option."""
        docs_dir = Path(mkdocs_config["docs_dir"])
        site_dir = Path(mkdocs_config["site_dir"])

        # Create compact HTML
        html_file = site_dir / "test.html"
        html_file.write_text("""<html><body><main><h1>Title</h1><p>Content</p></main></body></html>""")

        plugin.on_config(mkdocs_config)
        plugin.source_files["test.md"] = {
            "frontmatter": {},
            "abs_src_path": str(temp_dir / "test.md"),
        }
        
        # Test with prettify=True
        plugin.config["prettify"] = True
        plugin.on_post_build(mkdocs_config)
        
        stage_dir = docs_dir.parent / "stage"
        cousin_file = stage_dir / "test.md"
        content = cousin_file.read_text()
        
        # Should have proper indentation
        lines = content.strip().split('\n')
        assert len(lines) > 1  # Multiple lines
        assert any(" <" in line for line in lines)  # Has indentation

    def test_mutually_exclusive_options(self, plugin, mkdocs_config):
        """Test that minify and prettify are mutually exclusive."""
        plugin.config["minify"] = True
        plugin.config["prettify"] = True
        
        with pytest.raises(ValueError, match="Cannot use both 'minify' and 'prettify'"):
            plugin.on_config(mkdocs_config)



# this_file: more/mkdocs-plugins/vexy-mkdocs-output-as-input/src/mkdocs_output_as_input/plugin.py
"""MkDocs plugin that captures HTML output and creates cousin Markdown files."""

import logging
import re
from pathlib import Path
from typing import Any, Dict, Optional

import yaml
from bs4 import BeautifulSoup
from mkdocs.config import config_options
from mkdocs.plugins import BasePlugin
from mkdocs.structure.pages import Page

logger = logging.getLogger(__name__)


class OutputAsInputPlugin(BasePlugin):  # type: ignore[type-arg,no-untyped-call]
    """MkDocs plugin that captures HTML output and creates cousin Markdown files.

    This plugin:
    1. Lets the entire MkDocs build process run normally
    2. Tracks all source Markdown files and their YAML frontmatter
    3. After the build, creates a "stage" directory that replicates the source structure
    4. For each source Markdown file, creates a cousin file with original frontmatter
       and extracted HTML
    """

    config_scheme = (
        ("stage_dir", config_options.Type(str, default="stage")),
        ("html_element", config_options.Type((str, list), default="main")),
        ("target_tag", config_options.Type(str, default="article")),
        ("include_frontmatter", config_options.Type(bool, default=True)),
        ("preserve_links", config_options.Type(bool, default=False)),
        ("minify", config_options.Type(bool, default=False)),
        ("prettify", config_options.Type(bool, default=False)),
        ("verbose", config_options.Type(bool, default=False)),
    )

    def __init__(self) -> None:
        """Initialize the plugin."""
        super().__init__()
        self.source_files: Dict[str, Dict[str, Any]] = {}
        self.site_dir: Optional[Path] = None
        self.docs_dir: Optional[Path] = None

    def on_config(self, config: dict[str, Any]) -> dict[str, Any]:  # type: ignore[override]
        """Store site and docs directories and validate configuration."""
        self.site_dir = Path(config["site_dir"])
        self.docs_dir = Path(config["docs_dir"])

        # Validate mutually exclusive options
        if self.config["minify"] and self.config["prettify"]:
            raise ValueError("OutputAsInput: Cannot use both 'minify' and 'prettify' options")

        if self.config["verbose"]:
            logger.info(f"OutputAsInput: site_dir={self.site_dir}, docs_dir={self.docs_dir}")

        return config

    def on_page_read_source(self, page: Page, config: dict[str, Any]) -> Optional[str]:  # type: ignore[override]  # noqa: ARG002
        """Capture source Markdown content and frontmatter."""
        src_path = page.file.src_path

        # Read the full source content
        try:
            with open(page.file.abs_src_path, encoding="utf-8") as f:  # type: ignore[arg-type]
                content = f.read()
        except Exception as e:
            logger.error(f"OutputAsInput: Failed to read {src_path}: {e}")
            return None

        # Extract frontmatter if present
        frontmatter: Dict[str, Any] = {}
        if content.startswith("---\n"):
            try:
                end_idx = content.find("\n---\n", 4)
                if end_idx > 0:
                    fm_text = content[4:end_idx]
                    frontmatter = yaml.safe_load(fm_text) or {}
            except yaml.YAMLError as e:
                logger.warning(f"OutputAsInput: Failed to parse frontmatter in {src_path}: {e}")

        self.source_files[src_path] = {
            "frontmatter": frontmatter,
            "abs_src_path": page.file.abs_src_path,
        }

        if self.config["verbose"]:
            logger.info(f"OutputAsInput: Captured source {src_path}")

        return None  # Let MkDocs continue with original content

    def on_post_build(self, config: dict[str, Any]) -> None:  # type: ignore[override]  # noqa: ARG002
        """After build, process all HTML files and create cousin Markdowns."""
        if self.docs_dir is None:
            logger.error("OutputAsInput: docs_dir not set")
            return
        stage_dir = self.docs_dir.parent / self.config["stage_dir"]

        # Create stage directory
        stage_dir.mkdir(exist_ok=True)
        logger.info(f"OutputAsInput: Creating stage directory at {stage_dir}")

        # Process each tracked source file
        processed = 0
        for src_path, file_info in self.source_files.items():
            try:
                self._process_file(src_path, file_info, stage_dir)
                processed += 1
            except Exception as e:
                logger.error(f"OutputAsInput: Failed to process {src_path}: {e}")

        logger.info(f"OutputAsInput: Processed {processed}/{len(self.source_files)} files")

    def _process_file(self, src_path: str, file_info: dict[str, Any], stage_dir: Path) -> None:
        """Process a single file: extract HTML and create cousin Markdown."""
        if self.site_dir is None:
            logger.error("OutputAsInput: site_dir not set")
            return
        
        # Determine HTML output path
        # Special case: README.md often becomes index.html at root
        if src_path.lower() == "readme.md":
            html_path = "index.html"
            full_html_path = self.site_dir / html_path
        else:
            html_path = src_path.replace(".md", "/index.html")
            full_html_path = self.site_dir / html_path

        if not full_html_path.exists():
            # Try without the index.html suffix
            html_path = src_path.replace(".md", ".html")
            full_html_path = self.site_dir / html_path

        if not full_html_path.exists():
            logger.warning(f"OutputAsInput: No HTML output found for {src_path}")
            logger.warning(f"OutputAsInput: Looked for: {full_html_path}")
            return

        # Read and parse HTML
        try:
            with open(full_html_path, encoding="utf-8") as f:
                html_content = f.read()
        except Exception as e:
            logger.error(f"OutputAsInput: Failed to read HTML {full_html_path}: {e}")
            return

        soup = BeautifulSoup(html_content, "html.parser")

        # Extract target element(s)
        html_elements = self.config["html_element"]
        if isinstance(html_elements, str):
            html_elements = [html_elements]
        
        extracted_elements = []
        for selector in html_elements:
            # Try CSS selector first
            elements = soup.select(selector)
            if elements:
                extracted_elements.extend(elements)
            else:
                # Fall back to tag name
                element = soup.find(selector)
                if element:
                    extracted_elements.append(element)
        
        if not extracted_elements:
            logger.warning(
                f"OutputAsInput: No elements matching {self.config['html_element']} found in {full_html_path}"
            )
            return

        # If multiple elements, wrap them in a container
        if len(extracted_elements) > 1:
            container = soup.new_tag(self.config["target_tag"])
            for elem in extracted_elements:
                container.append(elem.extract())
            target_element = container
        else:
            target_element = extracted_elements[0]
            # Transform to target tag if different and single selector was a tag name
            if isinstance(self.config["html_element"], str) and self.config["target_tag"] != self.config["html_element"]:
                target_element.name = self.config["target_tag"]  # type: ignore[misc]

        # Handle link preservation if requested
        if self.config["preserve_links"]:
            # Convert absolute links back to relative
            for link in target_element.find_all(["a", "img", "link", "script"]):
                for attr in ["href", "src"]:
                    if link.has_attr(attr):
                        url = link[attr]
                        # Simple heuristic: if it starts with /, it's likely a root-relative link
                        # In a real implementation, this would be more sophisticated
                        if url.startswith("/") and not url.startswith("//"):
                            link[attr] = f".{url}"

        # Create cousin file path
        cousin_path = stage_dir / src_path
        cousin_path.parent.mkdir(parents=True, exist_ok=True)

        # Write cousin file
        try:
            with open(cousin_path, "w", encoding="utf-8") as f:
                # Write frontmatter if present and configured to include it
                if self.config["include_frontmatter"] and file_info["frontmatter"]:
                    f.write("---\n")
                    f.write(yaml.safe_dump(file_info["frontmatter"], default_flow_style=False))
                    f.write("---\n\n")

                # Write extracted HTML with formatting options
                if self.config["minify"]:
                    # Remove extra whitespace between tags
                    html_output = str(target_element)
                    # Remove newlines and compress multiple spaces
                    html_output = html_output.replace("\n", "")
                    html_output = re.sub(r'\s+', ' ', html_output)
                    # Remove spaces between tags
                    html_output = re.sub(r'>\s+<', '><', html_output)
                elif self.config["prettify"]:
                    html_output = target_element.prettify()
                else:
                    html_output = str(target_element)
                
                f.write(html_output)
                f.write("\n")

            if self.config["verbose"]:
                logger.info(f"OutputAsInput: Created cousin file {cousin_path}")

        except Exception as e:
            logger.error(f"OutputAsInput: Failed to write cousin file {cousin_path}: {e}")



"""CLI tool for standalone processing of HTML files with output-as-input plugin."""

import argparse
import logging
import sys
from pathlib import Path
from typing import Any, Dict, Optional

import yaml
from bs4 import BeautifulSoup

logger = logging.getLogger(__name__)


def setup_logging(verbose: bool = False) -> None:
    """Set up logging configuration."""
    level = logging.DEBUG if verbose else logging.INFO
    logging.basicConfig(
        level=level,
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )


def extract_frontmatter(content: str) -> tuple[Dict[str, Any], str]:
    """Extract YAML frontmatter from Markdown content.
    
    Args:
        content: The markdown content
        
    Returns:
        Tuple of (frontmatter dict, content without frontmatter)
    """
    frontmatter: Dict[str, Any] = {}
    body = content
    
    if content.startswith("---\n"):
        try:
            end_idx = content.find("\n---\n", 4)
            if end_idx > 0:
                fm_text = content[4:end_idx]
                frontmatter = yaml.safe_load(fm_text) or {}
                body = content[end_idx + 5:]  # Skip past the closing ---\n
        except yaml.YAMLError as e:
            logger.warning(f"Failed to parse frontmatter: {e}")
    
    return frontmatter, body


def process_html(
    html_content: str,
    html_element: str | list[str] = "main",
    target_tag: str = "article",
    preserve_links: bool = False,
    minify: bool = False,
    prettify: bool = False,
) -> Optional[str]:
    """Process HTML content and extract specified element.
    
    Args:
        html_content: The HTML content to process
        html_element: CSS selector for element to extract
        target_tag: Tag to replace the extracted element with
        preserve_links: Whether to preserve relative links
        minify: Whether to minify the output
        prettify: Whether to prettify the output
        
    Returns:
        Processed HTML string or None if element not found
    """
    soup = BeautifulSoup(html_content, "html.parser")
    
    # Extract target element(s)
    html_elements = html_element if isinstance(html_element, list) else [html_element]
    
    extracted_elements = []
    for selector in html_elements:
        # Try CSS selector first
        elements = soup.select(selector)
        if elements:
            extracted_elements.extend(elements)
        else:
            # Fall back to tag name
            element = soup.find(selector)
            if element:
                extracted_elements.append(element)
    
    if not extracted_elements:
        logger.error(f"No elements matching {html_element} found in HTML")
        return None
    
    # If multiple elements, wrap them in a container
    if len(extracted_elements) > 1:
        container = soup.new_tag(target_tag)
        for elem in extracted_elements:
            container.append(elem.extract())
        target_element = container
    else:
        target_element = extracted_elements[0]
        # Transform to target tag if different
        # For single selector, check if it's not a CSS selector
        if target_tag != target_element.name:
            # Only transform if original selector was a simple tag name
            if isinstance(html_element, str):
                # Simple tag name check
                if html_element == target_element.name:
                    target_element.name = target_tag
            elif len(html_element) == 1 and html_element[0] == target_element.name:
                target_element.name = target_tag
    
    # Handle link preservation if requested
    if preserve_links:
        # Convert absolute links back to relative
        for link in target_element.find_all(["a", "img", "link", "script"]):
            for attr in ["href", "src"]:
                if link.has_attr(attr):
                    url = link[attr]
                    # Simple heuristic: if it starts with /, it's likely a root-relative link
                    # In a real implementation, this would be more sophisticated
                    if url.startswith("/") and not url.startswith("//"):
                        link[attr] = f".{url}"
    
    # Format output
    if minify:
        # Remove extra whitespace between tags
        result = str(target_element).replace("\n", "").replace("  ", " ")
    elif prettify:
        result = target_element.prettify()
    else:
        result = str(target_element)
    
    return result


def process_file(
    input_path: Path,
    output_path: Path,
    html_element: str | list[str] = "main",
    target_tag: str = "article",
    include_frontmatter: bool = True,
    preserve_links: bool = False,
    minify: bool = False,
    prettify: bool = False,
) -> None:
    """Process a single file.
    
    Args:
        input_path: Path to input HTML file
        output_path: Path to output Markdown file
        html_element: CSS selector for element to extract
        target_tag: Tag to replace the extracted element with
        include_frontmatter: Whether to include frontmatter in output
        preserve_links: Whether to preserve relative links
        minify: Whether to minify the output
        prettify: Whether to prettify the output
    """
    logger.info(f"Processing {input_path} -> {output_path}")
    
    # Read input file
    try:
        content = input_path.read_text(encoding="utf-8")
    except Exception as e:
        logger.error(f"Failed to read {input_path}: {e}")
        raise
    
    # Check if there's a corresponding markdown file for frontmatter
    frontmatter = {}
    if include_frontmatter:
        md_path = input_path.with_suffix(".md")
        if md_path.exists():
            try:
                md_content = md_path.read_text(encoding="utf-8")
                frontmatter, _ = extract_frontmatter(md_content)
                logger.debug(f"Extracted frontmatter from {md_path}")
            except Exception as e:
                logger.warning(f"Failed to read markdown file {md_path}: {e}")
    
    # Process HTML
    processed_html = process_html(
        content,
        html_element=html_element,
        target_tag=target_tag,
        preserve_links=preserve_links,
        minify=minify,
        prettify=prettify,
    )
    
    if processed_html is None:
        raise ValueError(f"Failed to process HTML from {input_path}")
    
    # Create output directory if needed
    output_path.parent.mkdir(parents=True, exist_ok=True)
    
    # Write output
    try:
        with output_path.open("w", encoding="utf-8") as f:
            # Write frontmatter if present and configured
            if include_frontmatter and frontmatter:
                f.write("---\n")
                f.write(yaml.safe_dump(frontmatter, default_flow_style=False))
                f.write("---\n\n")
            
            # Write processed HTML
            f.write(processed_html)
            f.write("\n")
            
        logger.info(f"Successfully wrote {output_path}")
    except Exception as e:
        logger.error(f"Failed to write {output_path}: {e}")
        raise


def main() -> int:
    """Main CLI entry point."""
    parser = argparse.ArgumentParser(
        description="Process HTML files with MkDocs Output as Input plugin logic",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
  # Process a single file
  mkdocs-output-as-input process input.html output.md
  
  # Process with custom element and tag
  mkdocs-output-as-input process input.html output.md --html-element div --target-tag section
  
  # Process without frontmatter
  mkdocs-output-as-input process input.html output.md --no-frontmatter
  
  # Process with link preservation and prettify
  mkdocs-output-as-input process input.html output.md --preserve-links --prettify
        """,
    )
    
    subparsers = parser.add_subparsers(dest="command", help="Available commands")
    
    # Process command
    process_parser = subparsers.add_parser("process", help="Process HTML file(s)")
    process_parser.add_argument("input", type=Path, help="Input HTML file")
    process_parser.add_argument("output", type=Path, help="Output Markdown file")
    process_parser.add_argument(
        "--html-element",
        action="append",
        help="HTML element(s) to extract (can be specified multiple times, default: main)",
    )
    process_parser.add_argument(
        "--target-tag",
        default="article",
        help="Target tag for output (default: article)",
    )
    process_parser.add_argument(
        "--no-frontmatter",
        action="store_false",
        dest="include_frontmatter",
        help="Exclude frontmatter from output",
    )
    process_parser.add_argument(
        "--preserve-links",
        action="store_true",
        help="Preserve relative links in HTML",
    )
    process_parser.add_argument(
        "--minify",
        action="store_true",
        help="Minify HTML output",
    )
    process_parser.add_argument(
        "--prettify",
        action="store_true",
        help="Prettify HTML output",
    )
    process_parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="Enable verbose logging",
    )
    
    args = parser.parse_args()
    
    if not args.command:
        parser.print_help()
        return 1
    
    # Set up logging
    setup_logging(args.verbose)
    
    try:
        if args.command == "process":
            # Validate mutually exclusive options
            if args.minify and args.prettify:
                logger.error("Cannot use both --minify and --prettify")
                return 1
            
            # Default to "main" if no elements specified
            html_element = args.html_element if args.html_element else "main"
            
            process_file(
                args.input,
                args.output,
                html_element=html_element,
                target_tag=args.target_tag,
                include_frontmatter=args.include_frontmatter,
                preserve_links=args.preserve_links,
                minify=args.minify,
                prettify=args.prettify,
            )
            return 0
    except Exception as e:
        logger.error(f"Error: {e}")
        return 1
    
    return 0


if __name__ == "__main__":
    sys.exit(main())


"""Tests for the CLI tool."""

import tempfile
from pathlib import Path
from unittest.mock import patch

import pytest

from mkdocs_output_as_input.cli import (
    extract_frontmatter,
    main,
    process_file,
    process_html,
)


class TestCLI:
    """Test cases for CLI functionality."""

    def test_extract_frontmatter_with_frontmatter(self):
        """Test extracting YAML frontmatter from content."""
        content = """---
title: Test Title
author: Test Author
---

# Content here
"""
        frontmatter, body = extract_frontmatter(content)
        
        assert frontmatter == {"title": "Test Title", "author": "Test Author"}
        assert body.strip() == "# Content here"

    def test_extract_frontmatter_without_frontmatter(self):
        """Test extracting from content without frontmatter."""
        content = "# Just content"
        frontmatter, body = extract_frontmatter(content)
        
        assert frontmatter == {}
        assert body == "# Just content"

    def test_extract_frontmatter_invalid_yaml(self):
        """Test handling invalid YAML frontmatter."""
        content = """---
title: Test
invalid: [unclosed
---

# Content
"""
        frontmatter, body = extract_frontmatter(content)
        
        # Should handle gracefully
        assert frontmatter == {}
        assert body == content

    def test_process_html_basic(self):
        """Test basic HTML processing."""
        html = """<!DOCTYPE html>
<html>
<body>
<main class="content">
<h1>Title</h1>
<p>Content</p>
</main>
</body>
</html>"""
        
        result = process_html(html)
        
        assert result is not None
        assert '<article class="content">' in result
        assert '<h1>Title</h1>' in result
        assert '<p>Content</p>' in result

    def test_process_html_custom_element_and_tag(self):
        """Test processing with custom element and tag."""
        html = """<html>
<body>
<div id="content">
<h1>Title</h1>
</div>
</body>
</html>"""
        
        result = process_html(html, html_element="div", target_tag="section")
        
        assert result is not None
        assert '<section id="content">' in result
        assert '<h1>Title</h1>' in result

    def test_process_html_element_not_found(self):
        """Test handling when element not found."""
        html = "<html><body><p>No main element</p></body></html>"
        
        result = process_html(html)
        
        assert result is None

    def test_process_html_preserve_links(self):
        """Test preserving relative links."""
        html = """<html>
<body>
<main>
<a href="/docs/page.html">Link</a>
<img src="/images/pic.png">
<a href="https://example.com">External</a>
</main>
</body>
</html>"""
        
        result = process_html(html, preserve_links=True)
        
        assert result is not None
        assert 'href="./docs/page.html"' in result
        assert 'src="./images/pic.png"' in result
        assert 'href="https://example.com"' in result  # External unchanged

    def test_process_html_minify(self):
        """Test HTML minification."""
        html = """<html>
<body>
<main>
    <h1>Title</h1>
    <p>Content</p>
</main>
</body>
</html>"""
        
        result = process_html(html, minify=True)
        
        assert result is not None
        assert "\n" not in result
        assert "  " not in result

    def test_process_html_prettify(self):
        """Test HTML prettification."""
        html = "<html><body><main><h1>Title</h1><p>Content</p></main></body></html>"
        
        result = process_html(html, prettify=True)
        
        assert result is not None
        assert "\n" in result
        assert " <h1>" in result  # Indented

    def test_process_file_basic(self, tmp_path):
        """Test basic file processing."""
        # Create input HTML file
        input_file = tmp_path / "input.html"
        input_file.write_text("""<html>
<body>
<main>
<h1>Test Page</h1>
</main>
</body>
</html>""")
        
        output_file = tmp_path / "output.md"
        
        process_file(input_file, output_file)
        
        assert output_file.exists()
        content = output_file.read_text()
        assert "<article>" in content
        assert "<h1>Test Page</h1>" in content

    def test_process_file_with_frontmatter(self, tmp_path):
        """Test processing with frontmatter from corresponding .md file."""
        # Create input HTML file
        input_file = tmp_path / "page.html"
        input_file.write_text("<html><body><main><h1>Page</h1></main></body></html>")
        
        # Create corresponding markdown file with frontmatter
        md_file = tmp_path / "page.md"
        md_file.write_text("""---
title: Test Page
author: Test Author
---

# Original content
""")
        
        output_file = tmp_path / "output.md"
        
        process_file(input_file, output_file)
        
        content = output_file.read_text()
        assert "---" in content
        assert "title: Test Page" in content
        assert "author: Test Author" in content
        assert "<article>" in content

    def test_process_file_no_frontmatter(self, tmp_path):
        """Test processing without frontmatter."""
        input_file = tmp_path / "input.html"
        input_file.write_text("<html><body><main><h1>Page</h1></main></body></html>")
        
        output_file = tmp_path / "output.md"
        
        process_file(input_file, output_file, include_frontmatter=False)
        
        content = output_file.read_text()
        assert not content.startswith("---")
        assert "<article>" in content

    def test_process_file_creates_output_dir(self, tmp_path):
        """Test that output directory is created if needed."""
        input_file = tmp_path / "input.html"
        input_file.write_text("<html><body><main><h1>Page</h1></main></body></html>")
        
        output_file = tmp_path / "subdir" / "output.md"
        
        process_file(input_file, output_file)
        
        assert output_file.exists()
        assert output_file.parent.exists()

    def test_cli_process_command(self, tmp_path):
        """Test CLI process command."""
        input_file = tmp_path / "input.html"
        input_file.write_text("<html><body><main><h1>Test</h1></main></body></html>")
        
        output_file = tmp_path / "output.md"
        
        with patch("sys.argv", ["mkdocs-output-as-input", "process", 
                               str(input_file), str(output_file)]):
            result = main()
        
        assert result == 0
        assert output_file.exists()

    def test_cli_custom_options(self, tmp_path):
        """Test CLI with custom options."""
        input_file = tmp_path / "input.html"
        input_file.write_text("<html><body><div id='content'><h1>Test</h1></div></body></html>")
        
        output_file = tmp_path / "output.md"
        
        with patch("sys.argv", [
            "mkdocs-output-as-input", "process",
            str(input_file), str(output_file),
            "--html-element", "div",
            "--target-tag", "section",
            "--no-frontmatter",
            "--preserve-links",
            "--prettify",
            "--verbose"
        ]):
            result = main()
        
        assert result == 0
        content = output_file.read_text()
        assert "<section" in content
        assert not content.startswith("---")

    def test_cli_mutually_exclusive_options(self, tmp_path):
        """Test that minify and prettify are mutually exclusive."""
        input_file = tmp_path / "input.html"
        input_file.write_text("<html><body><main><h1>Test</h1></main></body></html>")
        
        output_file = tmp_path / "output.md"
        
        with patch("sys.argv", [
            "mkdocs-output-as-input", "process",
            str(input_file), str(output_file),
            "--minify", "--prettify"
        ]):
            result = main()
        
        assert result == 1  # Should fail

    def test_cli_no_command(self):
        """Test CLI with no command shows help."""
        with patch("sys.argv", ["mkdocs-output-as-input"]):
            result = main()
        
        assert result == 1

    def test_cli_file_not_found(self, tmp_path):
        """Test CLI with non-existent input file."""
        input_file = tmp_path / "nonexistent.html"
        output_file = tmp_path / "output.md"
        
        with patch("sys.argv", ["mkdocs-output-as-input", "process",
                               str(input_file), str(output_file)]):
            result = main()
        
        assert result == 1